// Sample RFM69 sender/node, with ACK and encryption
// Sends periodic messages to the gateway (id=1)
// Get the RFM69 and SPIFlash library at: https://github.com/LowPowerLab/
#include <RFM69.h>
#include <SPI.h>
//#include <SPIFlash.h>
#define GATEWAYID     1   
#define NODEID        2    //unique for each node on same network

#define NETWORKID     55   //the same on all nodes that talk to each other

//Match frequency to the country (uncomment one):
//#define FREQUENCY     RF69_433MHZ
//#define FREQUENCY     RF69_868MHZ
#define FREQUENCY     RF69_915MHZ
#define CRYPTKEY      "crosspollination" //exactly the same 16 characters on all nodes
#define ACK_TIME      30 // max # of ms to wait for an ack

#define LED           13 // LED on D13
#define FLASH_SS      8  // FLASH SS on D8

#define SERIAL_BAUD   115200

int TRANSMITPERIOD = 400; //transmit a packet to gateway so often (in ms)
char payload[] = "This is Node ";
char buff[20];
byte sendSize=0;
boolean requestACK = false;

RFM69 radio;

void setup()
{
  Serial.begin(SERIAL_BAUD);
  
  radio.initialize(FREQUENCY,NODEID,NETWORKID);
  radio.encrypt(CRYPTKEY);

  char buff[50];
  sprintf(buff, "\nTransmitting on %d MHz.", FREQUENCY==RF69_868MHZ ? 868 : 915);
  Serial.println(buff);
}
  
long lastPeriod = -1;
void loop() {
  //process any serial input
  if (Serial.available() > 0){
    char input = Serial.read();
    if (input >= 48 && input <= 57) //[0,9]
    {
      TRANSMITPERIOD = 100 * (input-48);
      if (TRANSMITPERIOD == 0) TRANSMITPERIOD = 1000;
      Serial.print("\nChanging delay to ");
      Serial.print(TRANSMITPERIOD);
      Serial.println("ms\n");
    }

    if (input == 'i')
    {
      Serial.print("Node Number: ");
      word node = NODEID;
      Serial.println(node, DEC);
    }
  }

  //check for any received packets
  if (radio.receiveDone())
  {
    Serial.print('[');Serial.print(radio.SENDERID, DEC);Serial.print("] ");
    for (byte i = 0; i < radio.DATALEN; i++)
      Serial.print((char)radio.DATA[i]);
    Serial.print("   [RX_RSSI:");Serial.print(radio.RSSI);Serial.print("]");

    if (radio.ACKRequested())
    {
      radio.sendACK();
      Serial.print(" - ACK sent");
    }
    Serial.println();
  }

  int sendLength = 13;
  int currPeriod = (millis()+10*NODEID)/TRANSMITPERIOD;    // Have different node transmit at slightly different times.
  if (currPeriod != lastPeriod)
  {
    lastPeriod=currPeriod;
    Serial.print("Sending[");
    Serial.print("]: ");
    for(byte i = 0; i < sendLength; i++){
      Serial.print((char)payload[i]);
    }
    Serial.print( NODEID ) ;

    if (radio.sendWithRetry(GATEWAYID, payload, sendSize)){
       Serial.print(" Ack received.");
       }
    else{ 
       Serial.print(" No reply.");
    }
    Serial.println();
  }
}


